#include "config.h"
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
//#include <util/delay.h>
#include "mmc.h"


#define HARDWARE_SPI      0  // Not yet tested, not for 2313-platform

#define MMC_Write    SD_PORT
#define MMC_Read     SD_READPORT

#define MMC_CS_HIGH      MMC_Write |=  (1<<SPI_CS)
#define MMC_CS_LOW       MMC_Write &= ~(1<<SPI_CS)

extern uint8_t aucMmcBuf[MAX_RD_SIZE]; // 


#if UART_TESTOUT
  extern void usart_write_p_str( const char *Buffer );
  extern void vSendUartHexByte( uint8_t uc);
  extern void usart_write_char(char c);
#else
  #define  usart_write_p_str( x )
  #define  vSendUartHexByte( uc) 
  #define  usart_write_char( c )
#endif

#if MMC_MULTIBLOCK_READ
 #define CMD  aucMmcBuf    // With Multiblock-Read, we dont need to take account of current Sector-Adr, so re-use the sample-buffer for initial commands
#else
  uint8_t     CMD[6];      // global command register / Absolute sector counter
#endif
//uint16_t uiRdSectorCnt; // counts the bytes until sector is complete 0..511
register uint16_t uiRdSectorCnt asm("r4");  // r4/r5  -32
register uint16_t uiSectCntLeftL asm("r6"); // How many sector counts are left until file is over? Here: max = 65535*512 = 33.5 MB  -> Duration at 16kHz = 2100s = 34 Minutes
#if MMC_BIG_FILES
register uint8_t ucSectCntLeftH asm("r8");          // 33.5M * 256 = 8.7 GB, more than FAT16 can offer ( 2GB )  if you try to do this with a single long var, it takes much more flash
#endif


uint8_t spi_trans(uint8_t spibyte) 
{
#if HARDWARE_SPI
  SPDR = spibyte;
  while (!(SPSR & (1 << SPIF)));
  spibyte = SPDR;
#else
  
  for (uint8_t i = 8; i; i--)   // sämtliche do/while varianten sind nicht besser
  {
    if (spibyte & 0x80) 
    {
      MMC_Write |= (1 << SPI_MOSI);
    }
    else 
    {
      MMC_Write &= ~(1 << SPI_MOSI);
    }
    spibyte <<= 1;
    MMC_Write |= (1 << SPI_Clock);             // using the Bit-Toggle function needs more flash
    //asm("nop");                              // Pause wird für SD und 8MHz Proz nicht benötigt
    if( MMC_Read & (1<<SPI_MISO)) spibyte |= 1;
    MMC_Write &= ~(1 << SPI_Clock);
  }
#endif
  return spibyte;
}

#define spi_write_byte(x)  spi_trans(x)
#define spi_read_byte()    spi_trans(0xff)

//#if ...        spart 4 Bytes in Minimal-Konfig, nicht getestet, ob es sich bei anderer Config negativ auswirkt
inline 
//#endif 
uint8_t ucWaitSpiFEWithTimeout( void ) 
{
  uint8_t timeout = 255;  
  
  while( spi_read_byte() != 0xfe )
  {
    if( --timeout == 0 )
    {
      return 1;  // no success
    }
  }
 
  return 0;
}


//############################################################################
//Sendet ein Commando an die MMC/SD-Karte
uint8_t Write_Command_MMC ( void )
//############################################################################
{
  uint8_t a;
  uint8_t tmp;
  uint8_t Timeout = 255;
  uint8_t * pucCmd = CMD;

  //set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv) 
  MMC_CS_HIGH;
  //sends 8 clock pulses
  spi_write_byte(0xFF);

  //set MMC_Chip_Select to low (MMC/SD-Karte Aktiv)
  MMC_CS_LOW;
  
  //usart_write_p_str( PSTR("Send command: "));
  //sendet 6 Byte Commando
  
  for( a=0; a<6; a++ )
  {
    //vSendUartHexByte( *pucCmd );
    spi_write_byte(*pucCmd++);
  }
  //usart_write_char('\n');

  //Wartet auf ein gültige Antwort von der MMC/SD-Karte
  
  do
  {
    tmp = spi_read_byte();
    if (--Timeout == 0 )
    {
      break; //Abbruch da die MMC/SD-Karte nicht Antwortet
    }
  }
  while( tmp == 0xff ); // wait for command to be accepted

  return tmp;
}


// Block party

void MMC_Read_Block_Cont( uint8_t *Buffer,uint8_t Bytes )
{

  //Lesen des Blocks (normal 512Bytes) von MMC/SD-Karte
  uiRdSectorCnt += Bytes;

  while( Bytes-- )
  {
    *Buffer++ = spi_read_byte();
  }

  if( uiRdSectorCnt >= 512 ) // Sector complete chunks?
  {
    
    uiRdSectorCnt = 0;
    // skip CRC-Bytes
    spi_read_byte();//CRC - Byte wird nicht ausgewertet
    spi_read_byte();//CRC - Byte wird nicht ausgewertet
    
#if MMC_MULTIBLOCK_READ
    // Wait for Data Ready of the next sector and go on without pushing CS high
    if( 
        uiSectCntLeftL
#if MMC_BIG_FILES
        || ucSectCntLeftH
#endif       
      ) // playing file, left init-phase of reading single sectors
    {
      ucWaitSpiFEWithTimeout();
    }
    else
#endif  
    {
      //set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv)
      MMC_CS_HIGH;                         //Setzt den Pin MMC_Chip_Select auf High Pegel
    }
  }
  //usart_write_p_str( PSTR("Read Block: All seems OK\n"));
}

//#if Konfig-Test  für minimal-konfig holt das 24 Bytes raus!
inline 
//#endif
void MMC_Read_Block_Start( uint8_t *Buffer,uint8_t Bytes )
//############################################################################
{	
  
/*  
  usart_write_p_str( PSTR("Entered Read Block Bytes Requested:"));
  vSendUartHexByte( Bytes );
  usart_write_char( '\n' );
*/

  //Sendet vorbereitetes Read-Commando an MMC/SD-Karte
  if( Write_Command_MMC() != 0)
  {
    usart_write_p_str( PSTR("Read Block: Command not recognized\n"));
    MMC_CS_HIGH;                         //Setzt den Pin MMC_Chip_Select auf High Pegel
    return;
  }
  
  uiRdSectorCnt = 0;
          
  //Wartet auf Start Byte von der MMC/SD-Karte (FEh/Start Byte)
  
  if( ucWaitSpiFEWithTimeout() )
  {
    usart_write_p_str( PSTR("Read Block: Timed out waiting for 0xfe\n"));
    MMC_CS_HIGH;                         //Setzt den Pin MMC_Chip_Select auf High Pegel
    return;
  }
  MMC_Read_Block_Cont( Buffer, Bytes );
}


//############################################################################
//Routine zum Vorbereiten des Lesens eines Blocks von der MMC/SD-Karte
void mmc_read_sector (uint32_t addr,uint8_t *Buffer, uint8_t size)
//############################################################################
{	
	//Commando 17 zum lesen eines Blocks von der MMC/SD - Karte  0x40 + 17(0x11) = 0x51
//	static uint8_t CMD[] = {0x51,0x00,0x00,0x00,0x00,0xFF}; 

// 
  
  /*Die Adressierung der MMC/SD-Karte wird in Bytes angegeben,
    addr wird von Blocks zu Bytes umgerechnet danach werden 
    diese in das Commando eingefügt*/
#if MMC_MULTIBLOCK_READ  
  if( size == 0xff )
  {
    CMD[0] = 0x52;  // Multiblock-Read  0x40 + 0x12 ( 18 )
    size=0;         // this read always starts with 0 bytes read, just prepares the first read access
  }
  else
#endif    
  {
    CMD[0] = 0x51;
  }
    
  addr = addr << 9; //addr = addr * 512
  
  CMD[1] = ((addr & 0xFF000000) >>24 );
  CMD[2] = ((addr & 0x00FF0000) >>16 );
  CMD[3] = ((addr & 0x0000FF00) >>8  );
#if MMC_MULTIBLOCK_READ  
  CMD[4] = 0x00;  // ist schon aus init und ändert sich nie, nur bei Multisector-Read
#endif  
  CMD[5] = 0xFF;
  MMC_Read_Block_Start( Buffer, size );
}

#if !MMC_MULTIBLOCK_READ
void mmc_inc_and_read_sector( uint8_t *Buffer, uint8_t size )
{
  CMD[3] += 2;
  if( CMD[3] == 0 )
  {
    if( ++CMD[2]==0 )
    {
      ++CMD[1];
    }
  }
  MMC_Read_Block_Start( Buffer, size );
}
#endif

/*  For debugging only
#if MMC_MULTIBLOCK_READ
uint8_t mmc_get_status( void )
{
  uint8_t uc;
  
  CMD[0] = 0x4d;
  CMD[1] = 0x00;
  CMD[2] = 0x00;
  CMD[3] = 0x00;
  CMD[4] = 0x00;
  CMD[5] = 0xff;

  uc = Write_Command_MMC();
  MMC_CS_HIGH;
  
  return uc ;
}
#endif
*/

#if UART_TESTOUT

  void vDumpUlong( ULONG ul )
  {
    uint8_t a,b,c,d;

    a= ul>>24;
    vSendUartHexByte( a );
    b= ul>>16;
    vSendUartHexByte( b );
    c= ul>>8;
    vSendUartHexByte( c );
    d= ul;
    vSendUartHexByte( d );
    
    usart_write_char('\n');
  }


  void vDumpBuffer( void )
  {
    uint8_t x,y; 
    
    for( x=0; x<MAX_RD_SIZE; x++ )
    {
      if( !(x & 0x0F) )
      {
        usart_write_char('\n');
        vSendUartHexByte( x );
        usart_write_p_str( PSTR(": ") );
      }
      y= aucMmcBuf[x];
      vSendUartHexByte( y );
      /*
      if( y >= 0x20 && y <=0x7e )
      {
        usart_write_char(y);
      }
      else
      {
        usart_write_char('.');
      }
      usart_write_char(' ');
      */    
    }
    usart_write_char('\n');
  }

#else
  #define vDumpUlong( ul )
  #define vDumpBuffer()
#endif


uint8_t ucGetNextChunk( void )
{
  uint8_t ucFlagDone=0;
  
  if( !uiRdSectorCnt ) // A new sector starts
  {
    usart_write_char('.');
#if MMC_BIG_FILES
    if( !uiSectCntLeftL ) // Low-Word zero?
    {
      if( ucSectCntLeftH ) // but high-words left
      {
        ucSectCntLeftH--;
        uiSectCntLeftL= 0xFFFF;
      }
      else // nothing left
      {
        ucFlagDone=1;
      }
    }
    else  // Low word is not yet zero
    {
      uiSectCntLeftL--;
    }
#else
    if( --uiSectCntLeftL == 0 ) // We're done here?
    {
      ucFlagDone=1;
    }
#endif
    if( ucFlagDone )
    {
      usart_write_char('!');
#if MMC_MULTIBLOCK_READ
      MMC_CS_HIGH;                         //MMC_Chip_Select finally goes up here in Multiblock-Read
#endif        

#if MMC_IDLE_MODE
      // Send card into idle mode
      MMC_init(1);
#endif      
      return 1;  // done
    }
  }
  MMC_Read_Block_Cont( aucMmcBuf + (uiRdSectorCnt & SECTOR_HALF_BIT_MSK), MAX_RD_SIZE/2 );
#if !MMC_MULTIBLOCK_READ    
  if( !uiRdSectorCnt ) // this Sector just finished?
  {
    mmc_inc_and_read_sector( (uint8_t *)0,0);  // prepare next sector read already
  }
#endif  
  return 0;
}

void vSkipBlocks( uint8_t cnt )  // Read out rest of the sector, so card will be ready for next data read. This is really necessary!
{
  do
  {
    MMC_Read_Block_Cont( aucMmcBuf, MAX_RD_SIZE );          // Read out rest of the sector, so card will be ready for next data read
  }
  while( --cnt ); // while oben, oder warten auf RdByteCnt=0 kostet komischerweise mehr Code
}

#if MMC_NO_RETURN_CODE
  #define ucMalFunct(x) _ucMalFunct()
  
  void _ucMalFunct( void )  // inline +2 Bytes in minimal config
  {
    MMC_CS_HIGH;                         //Setzt den Pin MMC_Chip_Select auf High Pegel
  }
#else
  uint8_t ucMalFunct( uint8_t ucRet )
  {
    MMC_CS_HIGH;                         //Setzt den Pin MMC_Chip_Select auf High Pegel
    return ucRet;
  }
#endif
  
//############################################################################
//Routine zur Initialisierung der MMC/SD-Karte (SPI-MODE)
#if MMC_NO_RETURN_CODE
       void 
#else
       uint8_t 
#endif       
                      MMC_init (          
#if MMC_IDLE_MODE
                                  uint8_t ucJustGoIdle 
#else
                                  void 
#endif
                                                         )
//############################################################################
{
  uint32_t ulTmp;
  uint16_t uiRootDirEntries;
  uint16_t uiClusterStartSector;
  uint8_t  Timeout=0;
  uint8_t  x;
  uint8_t  ucSectorsPerCluster;

  uiSectCntLeftL=0;  // This is the trigger for ucGetNextChunk to stop after reading the sector
                     // in case no valid data had been found on the card and no return function is used
#if MMC_BIG_FILES
  ucSectCntLeftH=0;
#endif

#if HARDWARE_SPI
  // Activate and configure hardware SPI at F_CPU/4
  SPCR |= (1 << SPE | 1 << MSTR);
#endif
  
  MMC_CS_HIGH;                         //Setzt den Pin MMC_Chip_Select auf High Pegel
  
  //Initialisiere MMC/SD-Karte in den SPI-Mode
  for( x=0; x<0x0f; x++) //Sendet min 74+ Clocks an die MMC/SD-Karte
  {
    spi_write_byte(0xff);
  }
  
  //Sendet Commando CMD0 "go to idle" an MMC/SD-Karte
  CMD[0] = 0x40;
  CMD[1] = 0x00;
  CMD[2] = 0x00;
  CMD[3] = 0x00;
  CMD[4] = 0x00;
  CMD[5] = 0x95;
  
  while( Write_Command_MMC() !=1 )
  {
    if( --Timeout==0 )
    {
      return ucMalFunct(1);
    }
  }

  usart_write_p_str( PSTR("Timeout0 was "));
  vSendUartHexByte( Timeout );
  usart_write_char('\n');


#if MMC_IDLE_MODE
  if( ucJustGoIdle )
  {
    return ucMalFunct(0); // no malfunct actually
  }
#endif  

  //Sendet Commando CMD1 an MMC/SD-Karte, leaves idle-mode
  Timeout = 255;
  CMD[0] = 0x41;//Commando 1
  CMD[5] = 0xFF;
  while( Write_Command_MMC() !=0 )
  {
    if( --Timeout==0 )
    {
      usart_write_p_str( PSTR("Answer to CMD1 was not 0\n"));
      usart_write_char('\n');
      return ucMalFunct(2);
    }
  }
  
/*  
  usart_write_p_str( PSTR("Timeout1 was "));
  vSendUartHexByte( Timeout );
  usart_write_char('\n');
*/  
  
  //set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv)
  MMC_CS_HIGH;                         //Setzt den Pin MMC_Chip_Select auf High Pegel

  // Take a look at the SD-Card now and find the WAV File
  
  // Read the mbr in order to find out where the honey is
  mmc_read_sector(0x0000000 ,aucMmcBuf, MAX_RD_SIZE );    // 0x000..0x3F    0
  vDumpBuffer();
  
#if MAX_RD_SIZE == 64
  vSkipBlocks( 7 );
/*    
  MMC_Read_Block_Cont( aucMmcBuf, MAX_RD_SIZE );          // 0x040..        1
  MMC_Read_Block_Cont( aucMmcBuf, MAX_RD_SIZE );          // 0x080..        2
  MMC_Read_Block_Cont( aucMmcBuf, MAX_RD_SIZE );          // 0x0C0..        3
  MMC_Read_Block_Cont( aucMmcBuf, MAX_RD_SIZE );          // 0x100..        4
  MMC_Read_Block_Cont( aucMmcBuf, MAX_RD_SIZE );          // 0x140..        5
  MMC_Read_Block_Cont( aucMmcBuf, MAX_RD_SIZE );          // 0x180..        6
  MMC_Read_Block_Cont( aucMmcBuf, MAX_RD_SIZE );          // 0x1C0..        7  // auto-ends MBR sector read operation with CS-HI
*/
#elif MAX_RD_SIZE == 32
  vSkipBlocks( 14 );  // 14 = 0x1c0..1df  15=1e0..1ff
#endif
 
#if FULL_SD_VALID_CHECK
  // would be possible with 32 Bytes Buf as well
  if( aucMmcBuf[ 0x1be - 0x1c0 + 4 ]  != 0x06 )         // 1c2
  {
    usart_write_p_str( PSTR("Not FAT16\n") );  
    vSendUartHexByte( aucMmcBuf[0x1be - 0x1c0 + 4] );
    usart_write_char('\n');
    return ucMalFunct(2);
  }
#endif  

  // Read absolute FAT start sector
  ulTmp = *(uint32_t *)(aucMmcBuf + 0x1be +  8 - 0x1c0 );

#if MAX_RD_SIZE == 32
  vSkipBlocks( 1 );  // 15=1e0..1ff, Sector done now
#endif

#if FULL_SD_VALID_CHECK
  if(  (aucMmcBuf[0x1fe - 0x1c0] != 0x55 ) || (aucMmcBuf[0x1ff - 0x1c0] != 0xAA ) )
  {
    usart_write_p_str( PSTR("FAT-signature missing\n") );
    vSendUartHexByte( aucMmcBuf[0x1fe - 0x1c0] );
    vSendUartHexByte( aucMmcBuf[0x1ff - 0x1c0] );
    usart_write_char('\n');
    //return ucMalFunct(4);
    // CS is already high
    return 4;
  }
#endif  


  usart_write_p_str( PSTR("\nBootsct of Partition at ") );
  vDumpUlong( ulTmp );  

  
  // Look at the Partition-Table
  mmc_read_sector(ulTmp ,aucMmcBuf, MAX_RD_SIZE );    // 0x000..0x3F    0
  vDumpBuffer();

  ucSectorsPerCluster =  aucMmcBuf[0x0d];  // Sectors per Cluster  ( 1 Byte )
  // 1GB-Karte:   0x20 = 32 Sectors/Cluster,   Clustersize = 32*512 = 16k

  usart_write_p_str( PSTR("\nSectors/Cluster ") );  
  vDumpUlong( ucSectorsPerCluster );
  
  uiRootDirEntries = *( uint16_t *)( aucMmcBuf + 0x11 );  // 1GB-Karte = 0x200 = 512 Einträge a 32 Bytes = 16k

  usart_write_p_str( PSTR("\nRoot-Entries "));
  vDumpUlong( uiRootDirEntries );

#if FULL_SD_VALID_CHECK
  if( *(uint16_t *)(aucMmcBuf +0x0b) != 0x0200 )
  {
    usart_write_p_str( PSTR("Sector-size in FAT is not 512 Bytes\n") );  
    return ucMalFunct(5);
  }
  
  if( aucMmcBuf[0x36+3] != '1' || aucMmcBuf[0x36+4] != '6' )    // "FAT16  "
  {
    usart_write_p_str( PSTR("Filesystem not FAT16\n") );
    return ucMalFunct(6);
  }
#endif

  // Skip to root-directory:
  ulTmp += *( uint16_t *)( aucMmcBuf + 0x0e ) + ( aucMmcBuf[0x10] * *( uint16_t *)( aucMmcBuf + 0x16 ) ); //   ulTmp += Reserved + ( NrOfFats * SectorsPerFat )
  
#if MAX_RD_SIZE == 64
  vSkipBlocks(7);
#elif MAX_RD_SIZE == 32
  vSkipBlocks(15);
#endif

  // Read root directory 
  
  usart_write_p_str( PSTR("Dir-start at ") );
  vDumpUlong( ulTmp );

  mmc_read_sector(ulTmp, aucMmcBuf, 0 );    // Position reader
  
  uiClusterStartSector = ulTmp+(uiRootDirEntries * 32)/512;   // Das ist der erste Sektor von Cluster 2.  Die fangen bei 2 an zu zählen, warum auch immer.
  
  usart_write_p_str( PSTR("\nCluster-Start-Sector at ") );  
  vDumpUlong( uiClusterStartSector );
  
  //vDumpBuffer();
  
  // FAT16 Direcotry Entries are 32 bytes big and organized like this:
  
  // Offset  0(8)  8(3)       11      12  13               14/15              16/17         18/19             20/21           22   24   26 27   28 29 30 31
  //         Name  Extension  Attrib  Res created time(ms) Created Time HHMM  created date  Last access date  Extended attrib Time Date Cluster File Size ( Bytes )

#if MMC_DIR_SCAN
  
  
  for( x=0; x<16; x++ )
  {
    // get next entry
 #if MAX_RD_SIZE == 32
    vSkipBlocks(1); // read next entry, we only have buffer-space for one entry
 #else
    MMC_Read_Block_Cont( aucMmcBuf, 32 );   
 #endif  
    vDumpBuffer();  

 #if FULL_SD_VALID_CHECK
    if( (aucMmcBuf[8] == 'W') || (aucMmcBuf[9] == 'A') || (aucMmcBuf[10] == 'V') ) // WAV not found..
 #else      
    if( aucMmcBuf[8] == 'W')
 #endif      
    {
      break;
    }
  }
    
  // if x isnt 16, we found a WAV file
 #if FULL_SD_VALID_CHECK
  if( x == 16 ) // no file found but sector read is complete, we can just leave
  {
    uiSectCntLeftL=1; // Make GetChunk stop immediately
   #if MMC_BIG_FILES    
    ucSectCntLeftH=0;
   #endif    
    return ucMalFunct(7);  // file not found
  }
 #endif
  
#else  // no dir scan, just pick the first entry

 #if MAX_RD_SIZE == 32
    vSkipBlocks(1); // skip first root-entry "." 
    vSkipBlocks(1); // read 2nd entry, this is the real file
 #else
    MMC_Read_Block_Cont( aucMmcBuf, 32 );   
    MMC_Read_Block_Cont( aucMmcBuf, 32 );   
 #endif  
  
#endif
    
  // use data of current entry    
  usart_write_p_str( PSTR("Wav!\n") );
  vDumpBuffer();  
  
#if MMC_BIG_FILES
  usart_write_p_str( PSTR("\nFile-Length:") );  
  ulTmp = *(uint32_t *)(aucMmcBuf + 28 );
  
  vDumpUlong( ulTmp  );
  ulTmp /=512;  

  //ulTmp = *(uint32_t *)(aucMmcBuf+28)/512;  // 0x1C,  File-Length in Bytes /512 -> coarse size in sectors
  
  uiSectCntLeftL = (uint16_t)ulTmp;  // Soviele Sektoren sollen bis zum Ende der WAV gelesen werden.
  ulTmp >>= 16;
  ucSectCntLeftH = (uint8_t)ulTmp;
  usart_write_p_str( PSTR("\nFile Sectors:") );  
  vDumpUlong( ucSectCntLeftH  );
  vDumpUlong( uiSectCntLeftL  );
  

  // 1 Abziehen zur Sicherheit, so dass am Ende kein Schrott mit eingesaugt wird.
  if( uiSectCntLeftL == 0 )
  {
    ucSectCntLeftH--;
    uiSectCntLeftL = 0xFFFF;
  }
  else
  {
    uiSectCntLeftL--;
  }
    
#else
 #if MMC_MAXPLAY
  uiSectCntLeftL = 0;  // Pre-Drecement, use full available if its there or not
 #else
  usart_write_p_str( PSTR("\nFile-Length:") );  
  vDumpUlong( *(uint32_t *)(aucMmcBuf+28) );
  
  ulTmp = *(uint32_t *)(aucMmcBuf+28)/512;  // 0x1C,  File-Length in Bytes /512 -> coarse size in sectors
  
  uiSectCntLeftL = (uint16_t)ulTmp-1;         // kein Limiter auf 64k Sektoren. Merkt der User dann selbst..
 #endif
#endif
  // First cluster (2) starts behind the directory stuff = Start Root dir + root-dir-size
  // Get absolute start sector of file
  ulTmp = ((*(uint16_t *)(aucMmcBuf+26) - 2) * ucSectorsPerCluster) + uiClusterStartSector + 1;
  
  usart_write_p_str( PSTR("\nFile-Output-Start-Sector at ") );  
  vDumpUlong( ulTmp );

  
  // read until end of sector, if necessary
#if MAX_RD_SIZE == 32
  vSkipBlocks(15-x); 
#else
  while( x<16 ) // Read until end of sector
  {
    MMC_Read_Block_Cont( aucMmcBuf, 32 );   
    x++;
  }
#endif  
    
  // Prepare File-Read.   The file may not be fragmented.  So best way to prepare an SD-Card is to
  // format it to FAT16 and place a single WAV File on it. This always worked here so far.

#if MMC_MULTIBLOCK_READ  
  //ulTmp     |= 0x80000000L; // Make this a multisector-read
  usart_write_char('#');
  mmc_read_sector(ulTmp ,(uint8_t *)0,0xFF);  // Prepare initial mulit-sector read
#else
  mmc_read_sector(ulTmp ,(uint8_t *)0,0);     // Prepare initial sector read
#endif
  
#if !MMC_NO_RETURN_CODE
  return 0;  // everything OK
#endif  
}

